from typing import Tuple

from attrs import define, field


@define(frozen=True)
class Task:
    """ Represents a Ticktick task.

    Attributes:
        title: The title of the task.
        status: The status of the task. 0: default/open, 2: done, -1: abandoned.
        ticktick_id: The ID of the task. For example: 6f8a2b3c4d5e1f09a7b6c8d9e0f2
        ticktick_etag: The unique identifier for each task. For example: 4b7n9t2q
        created_date: The date when the task was created in format YYYY-MM-DD.
        focus_time: The focus time of the task.
        deleted: The status of the task. 0: not deleted, 1: deleted.
        tags: The tags of the task as a list of strings.
        project_id: The ID of the project of the task. For example: 6f8a2b3c4d5e1f09a7b6c8d9e0f2
        timezone: The timezone of the task.
        due_date: The due date of the task in format YYYY-MM-DD.
    """
    title: str
    ticktick_id: str = field(eq=str.lower)
    ticktick_etag: str = field(eq=str.lower)
    created_date: str
    status: int = 0
    focus_time: float = 0
    deleted: int = 0
    tags: Tuple[str, ...] = ()
    project_id: str = field(default="", eq=str.lower)
    timezone: str = field(default="", eq=str.lower)
    due_date: str = field(default="", eq=str.lower)



import json
import time
from datetime import datetime
from typing import List

import pytest

from tickthon import Task, ExpenseLog
from tickthon import TicktickClient


@pytest.fixture(scope="module")
def ticktick_client(ticktick_info):
    return TicktickClient(ticktick_info["username"], ticktick_info["password"])


def test_get_project_list(ticktick_client, data_folder_path):
    expected_project_lists = {"test-project-1": "90as8dfa09sd8f90asdf809",
                              "test-project-2": "90as8dfa0as23dfsdf809",
                              "test-project-3": "90as8df23f2390asdf809",
                              "test-project-4": "90as823fw3sd8f90asdf809",
                              "test-project-5": "90as8dfa23432fdsasdf809",
                              "test-project-6": "90as8df324fds8f90asdf809",
                              "test-project-7": "90as8d234sdf9sd8f90asdf809"}
    with open(data_folder_path / "project_profiles.json") as f:
        lists = json.load(f)

    project_lists = ticktick_client._get_folder_lists(lists)

    assert project_lists == expected_project_lists


def test_get_active_tasks(ticktick_client):
    active_tasks = ticktick_client.get_active_tasks()

    assert len(active_tasks) > 0
    assert len(ticktick_client.all_active_tasks) >= len(active_tasks)


def test_get_completed_tasks(ticktick_client):
    completed_tasks = ticktick_client.get_completed_tasks()
    assert len(completed_tasks) > 0


def test_get_ideas(ticktick_client):
    id_idea = ticktick_client.create_task(Task(title="Idea: test idea", created_date="2099-09-09",
                                               ticktick_id="test-id", ticktick_etag="test-etag"))

    ideas = ticktick_client.get_ideas()

    assert len(ideas) > 0
    assert isinstance(ideas, List) and all(isinstance(i, Task) for i in ideas)
    time.sleep(3)
    ticktick_client.complete_task(ticktick_client.get_task(id_idea))


def test_get_expense_logs(ticktick_client):
    id_expense_log = ticktick_client.create_task(Task(title="$100 test expense log", created_date="2099-09-09",
                                                      ticktick_id="test-id", ticktick_etag="test-etag"))

    expense_logs = ticktick_client.get_expense_logs()

    assert len(expense_logs) > 0
    assert (isinstance(expense_logs, List) and
            all(isinstance(i[0], Task) and isinstance(i[1], ExpenseLog) for i in expense_logs))
    time.sleep(3)
    ticktick_client.complete_task(ticktick_client.get_task(id_expense_log))


def test_complete_task(ticktick_client):
    task_id = ticktick_client.create_task(Task(title="test-task", created_date="2099-09-09",
                                               ticktick_id="test-id", ticktick_etag="test-etag"))

    task_to_complete = ticktick_client.get_task(task_id)

    ticktick_client.complete_task(task_to_complete)

    completed_task = ticktick_client.get_task(task_to_complete.ticktick_id)
    assert completed_task.status == 2


def test_get_overall_focus_time(ticktick_client):
    date = datetime.now().strftime("%Y-%m-%d")
    focus_time = ticktick_client.get_overall_focus_time(date)

    assert isinstance(focus_time, float)
    assert focus_time >= 0

# TODO: Create a test for get_deleted_tasks
# TODO: Create a test for get_abandoned_tasks



import os
import json
import warnings
from typing import Dict

CHECKINS_START_DATE = 20221231


def check_ticktick_ids() -> bool:
    """Checks if the Ticktick ids are set in the environment variables."""
    return os.getenv("TICKTICK_IDS") is not None


def get_ticktick_ids() -> Dict[str, Dict[str, str]]:
    if check_ticktick_ids():
        return json.loads(os.getenv("TICKTICK_IDS", default=""))
    warnings.warn("Environment variable TICKTICK_IDS is not set.")
    return {}



import math

import attrs
from tickthon import Task, dict_to_task
from tickthon._task_utils import _get_focus_time, _get_task_date


def test_dict_to_task(dict_task):
    expected_task = Task(ticktick_id="60c8d7b1e9b80e0595353bc6",
                         ticktick_etag="muu17zqq",
                         created_date="2021-06-15",
                         status=0,
                         title="Automation tasks",
                         focus_time=0.1,
                         deleted=0,
                         tags=("test", "unit"),
                         project_id="5f30772022d478db3ad1a9c2",
                         timezone="America/Bogota",
                         due_date="2023-08-03",
                         )

    task = dict_to_task(dict_task)

    assert attrs.asdict(task) == attrs.asdict(expected_task)


def test_get_ticktick_focus_time():
    rawt_focus_time = {"focusSummaries": [{"pomoDuration": 0, "stopwatchDuration": 100},
                                          {"pomoDuration": 260, "stopwatchDuration": 0},
                                          ]}

    focus_time = _get_focus_time(rawt_focus_time)

    assert math.isclose(focus_time, 0.1)


def test_get_focus_time():
    raw_focus_time = {"focus_time": 0.1}

    focus_time = _get_focus_time(raw_focus_time)

    assert math.isclose(focus_time, 0.1)


def test_get_date():
    raw_task_date = "2023-08-03T19:15:00.000+0000"
    raw_timezone = "America/Bogota"

    task_date = _get_task_date(raw_timezone, raw_task_date)

    assert task_date == "2023-08-03"



import pytest

from tickthon import Task, ExpenseLog
from tickthon._task_utils import _parse_expense_log


@pytest.mark.parametrize("expense_title, expected_amount", [
    # Test with an integer expense amount.
    ("$99 Test expense log", 99),

    # Test with an integer expense amount.
    ("$99.9 Test expense log", 99.9),
])
def test_parse_expense_log(expense_title, expected_amount):
    expected_expense_log = ExpenseLog(date="9999-09-09", expense=expected_amount, product="Test expense log")
    raw_expense_log = Task(title=expense_title, due_date="9999-09-09", created_date="9999-09-09",
                           ticktick_id="test", ticktick_etag="test")

    expense_log = _parse_expense_log(raw_expense_log)

    assert expense_log == expected_expense_log


@pytest.mark.parametrize("expense_title", [
    # Test with empty expense.
    "$",

    # Test with empty expense amount.
    "$ Test expense log",

    # Test with empty expense amount without spaces.
    "$Test expense log",
])
def test_parse_wrong_expense_log(expense_title):
    raw_expense_log = Task(title=expense_title, due_date="9999-09-09", created_date="9999-09-09",
                           ticktick_id="test", ticktick_etag="test")

    expense_log = _parse_expense_log(raw_expense_log)

    assert expense_log is None



from enum import Enum
from typing import Optional
import secrets

from requests import Session, Response


class RequestTypes(Enum):
    """Types of requests that can be sent to the Ticktick API."""
    GET = "GET"
    POST = "POST"


class TicktickAPI:
    """Ticktick API client."""
    HOST = "https://api.ticktick.com"
    BASE_URL = "/api/v2"
    SIGNIN_URL = BASE_URL + "/user/signin?wc=True&remember=True"

    USER_AGENT = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:95.0) Gecko/20100101 Firefox/95.0"
    X_DEVICE_ = '{"platform":"web","os":"OS X","device":"Firefox 95.0","name":"unofficial api!","version":4531,' \
                '"id":"6490' + secrets.token_hex(10) + '","channel":"website","campaign":"","websocket":""}'

    HEADERS = {'User-Agent': USER_AGENT,
               'x-device': X_DEVICE_}

    def __init__(self, username: str, password: str):
        self.session = Session()
        self.session.headers.update({"Content-Type": "application/json",
                                     "User-Agent": self.USER_AGENT,
                                     "x-device": self.X_DEVICE_
                                     })
        self.auth_token = self.login(username, password)

    def login(self, user: str, password: str) -> str:
        """Logs into Ticktick and returns the authentication token.

        Args:
            user: Ticktick username
            password: Ticktick password

        Returns:
            Ticktick authentication token
        """
        payload = {
            "username": user,
            "password": password
        }

        response = self.post(self.SIGNIN_URL, data=payload, token_required=False)
        response.raise_for_status()

        return response.json()["token"]

    def _base_request(self, request_type: RequestTypes, url: str, data: Optional[dict] = None,
                      token_required: bool = True) -> Response:
        """Sends a request to the Ticktick API.

        Args:
            request_type: Type of request to send
            url: URL to send the request to
            data: Data to send in the request. Defaults to None.
            token_required: Whether the request requires an authentication token. Defaults to True.

        Returns:
            Response from the Ticktick API
        """
        if token_required:
            self.session.headers.update({
                "Authorization": f"Bearer {self.auth_token}",
                "Cookie": f"t={self.auth_token}"
            })

        return self.session.request(request_type.value, f"{self.HOST}{url}", json=data)

    def post(self, url: str, data: Optional[dict] = None, token_required: bool = True) -> Response:
        """Sends a POST request to the Ticktick API.

        Args:
            url: URL to send the request to
            data: Data to send in the request. Defaults to None.
            token_required: Whether the request requires an authentication token. Defaults to True.

        Returns:
            Response from the Ticktick API
        """

        response = self._base_request(RequestTypes.POST, url, data, token_required)
        return response

    def get(self, url: str, data: Optional[dict] = None, token_required: bool = True) \
            -> Response:
        """Sends a GET request to the Ticktick API.

        Args:
            url: URL to send the request to
            data: Data to send in the request. Defaults to None.
            token_required: Whether the request requires an authentication token. Defaults to True.

        Returns:
            Response from the Ticktick API
        """

        response = self._base_request(RequestTypes.GET, url, data, token_required)
        response.raise_for_status()

        return response
